//
//  RubyLabel.swift
//  RubyTest
//
//  Created by Gabriele Quatela on 18/04/2020.
//  Copyright © 2020 Gabriele Quatela. All rights reserved.
//

import UIKit

class RubyLabel: UILabel {

    
    // Only override draw() if you perform custom drawing.
    // An empty implementation adversely affects performance during animation.
    // ルビを表示
    override func draw(_ rect: CGRect) {
        var lineHeight: CGFloat { self.font.pointSize * 1.0 }
        //super.draw(rect)
        

        // context allows you to manipulate the drawing context (i'm setup to draw or bail out)
        guard let context: CGContext = UIGraphicsGetCurrentContext() else {
            return
        }

        // Flip the coordinate system
        context.textMatrix = CGAffineTransform.identity;
        context.translateBy(x: 0, y: self.bounds.size.height);
        context.scaleBy(x: 1.0, y: -1.0);

        let path = CGMutablePath()
        let newRect = CGRect(x: self.bounds.origin.x, y: self.bounds.origin.y, width: self.bounds.size.width, height: self.bounds.size.height + 10)
        path.addRect(self.bounds)

        guard let str = self.attributedText else { return }

//        // set font color
//        str.addAttribute(NSAttributedString.Key(rawValue: kCTForegroundColorAttributeName as String), value:UIColor.purple , range: NSMakeRange(0,str.length))
//        // set font name & size
//        let fontRef = UIFont.systemFont(ofSize: 20, weight: UIFont.Weight.bold)
//        str.addAttribute(NSAttributedString.Key(rawValue: kCTFontAttributeName as String), value: fontRef, range:NSMakeRange(0, str.length))

        let frameSetter = CTFramesetterCreateWithAttributedString(str)
        let frame = CTFramesetterCreateFrame(frameSetter, CFRangeMake(0,str.length), path, nil)
        // Check need for truncate tail
        if (CTFrameGetVisibleStringRange(frame).length as Int) < str.length {

            // Required truncate

            let linesNS: NSArray  = CTFrameGetLines(frame)
            let linesAO: [AnyObject] = linesNS as [AnyObject]
            var lines: [CTLine] = linesAO as! [CTLine]

            let boundingBoxOfPath = path.boundingBoxOfPath


            let lastCTLine = lines.removeLast()

            let truncateString:CFAttributedString = CFAttributedStringCreate(nil, "\u{2026}" as CFString, CTFrameGetFrameAttributes(frame))
            let truncateToken:CTLine = CTLineCreateWithAttributedString(truncateString)

            let lineWidth = CTLineGetTypographicBounds(lastCTLine, nil, nil, nil)
            let tokenWidth = CTLineGetTypographicBounds(truncateToken, nil, nil, nil)
            let widthTruncationBegins = lineWidth - tokenWidth
            if let truncatedLine = CTLineCreateTruncatedLine(lastCTLine, widthTruncationBegins, .end, truncateToken) {
                lines.append(truncatedLine)
            }

            var lineOrigins = Array<CGPoint>(repeating: CGPoint.zero, count: lines.count)
            CTFrameGetLineOrigins(frame, CFRange(location: 0, length: lines.count), &lineOrigins)
            for (index, line) in lines.enumerated() {
                context.textPosition = CGPoint(x: lineOrigins[index].x + boundingBoxOfPath.origin.x, y:lineOrigins[index].y + boundingBoxOfPath.origin.y)
                CTLineDraw(line, context)
            }
        }
        else {
            // Not required truncate
            CTFrameDraw(frame, context)
        }
    }
}
